import argparse
import asyncio
import sys
from datetime import datetime
from datetime import timezone
from types import FrameType

import uvicorn
import uvloop

from .controller import import_scrapers
from .controller import run_all

app_fastapi = None


class SnowScraper(uvicorn.Server):
    def handle_exit(self, sig: int, frame: FrameType) -> None:
        print("Shutting down SnowScraper")
        return super().handle_exit(sig, frame)


async def start_server() -> None:
    print("Starting SnowScraper")
    server = SnowScraper(config=uvicorn.Config(app_fastapi, workers=1, loop="uvloop"))
    fastapi = asyncio.create_task(server.serve())

    await asyncio.wait([fastapi], return_when=asyncio.FIRST_COMPLETED)


def main(args: argparse.Namespace) -> None:
    global app_fastapi

    from .webserver import app as app_fastapi

    import_scrapers()
    run_all(args)

    if sys.version_info >= (3, 11):
        with asyncio.Runner(loop_factory=uvloop.new_event_loop) as runner:
            runner.run(start_server())
    else:
        uvloop.install()
        asyncio.run(start_server())


def run():
    parser = argparse.ArgumentParser(description="Snowflake scraper")
    parser.add_argument(
        "--after",
        type=str,
        required=False,
        default=None,
        help="Scrape only after a specific date in the format 'MM-DD-YYYY' in UTC",
    )

    args = parser.parse_args()

    if args.after:
        try:
            args.after = datetime.strptime(args.after, "%m-%d-%Y").replace(tzinfo=timezone.utc)
        except ValueError:
            print(f"Error: The 'after' argument should be in the format MM-DD-YYYY. You provided: {args.after}")
            sys.exit(1)
    main(args)


if __name__ == "__main__":
    run()
